using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Security.Cryptography;
using System.IO;
using CSockets;
using CiaoStuff;

namespace CiaoServer
{
    public partial class MainServerForm : Form
    {
        /*Inject user32.dll to call scroll down*/
        [DllImport("user32.dll")]
        public static extern int SendMessage(IntPtr handle, int message, int wparam, int lparam);
        /*End injecting*/

        private FormWindowState PreviousWindowState;
        private ServerConfiguration ThisConfiguration;
        private List<User> UserList;
        private List<ConnectedClient> ClientList;
        private bool SaveFileDialogOkPressed;
        private Server CiaoServer;
        private Server ReportServer;

        public MainServerForm()
        {
            System.Windows.Forms.Form.CheckForIllegalCrossThreadCalls = false;
            InitializeComponent();
            LoadCofiguration("conf.xml");
            CiaoServer = new Server(3724, new PacketHandler(this.PacketHandling));
            ReportServer = new Server(3725, new PacketHandler(this.ReportPacketHandling));
            SaveFileDialogOkPressed = false;
            PreviousWindowState = this.WindowState;
            this.CiaoServer.ClientChange += new ClientEventHandler(CiaoServer_ClientChange);
            this.CiaoServer.PropertyChange += new PropertyChangedEventHandler(CiaoServer_PropertyChange);
            this.ReportServer.PropertyChange += new PropertyChangedEventHandler(ReportServer_PropertyChange);
            Log("Form initialized.");
            UserList = new List<User>(); LoadUserList("UserList.bin");
            UpdateUserListBox();
            ClientList = new List<ConnectedClient>();
        }

        void CiaoServer_PropertyChange(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsRunning")
            {
                if (CiaoServer.IsRunning)
                {
                    this.button1.Text = "Stop";
                }
                else
                {
                    this.button1.Text = "Start";
                }
            }
        }

        void ReportServer_PropertyChange(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsRunning")
            {
                if (ReportServer.IsRunning)
                {
                    this.button3.Text = "Stop";
                }
                else
                {
                    this.button3.Text = "Start";
                }
            }
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void menuItem4_Click(object sender, EventArgs e)
        {
            Options OptionsForm = new Options(ThisConfiguration);
            OptionsForm.ShowDialog(this);
            ThisConfiguration = OptionsForm.ThisConfiguration;
            Serializer.SaveObject(ThisConfiguration, "conf.xml", Serializer.FormatterType.SOAP);
        }

        private void MainForm_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized && this.ThisConfiguration.HideWhenMinimized)
            {
                this.TrayIcon.Visible = true;
                this.Visible = false;
            }
            else
            {
                this.PreviousWindowState = this.WindowState;
            }
        }

        private void notifyIcon1_Click(object sender, EventArgs e)
        {
            this.Visible = true;
            this.WindowState = this.PreviousWindowState;
            this.TrayIcon.Visible = false;
        }

        private void menuItem6_Click(object sender, EventArgs e)
        {
            Form AboutForm = new CiaoStuff.About(this.ProductName, this.ProductVersion);
            AboutForm.ShowDialog(this);
        }

        private void Log(String s)
        {
            this.textBox4.AppendText(DateTime.Now.ToString() + ": ");
            this.textBox4.AppendText(s + Environment.NewLine);
            SendMessage(this.textBox4.Handle, 277, 7, 0);
        }

        private void UpdateUserListBox()
        {
            this.listView1.Items.Clear();
            for (int i = 0; i < UserList.Count; i++)
            {
                String[] a = new String[5];
                a[0] = UserList[i].NickName;
                a[1] = UserList[i].eMail;
                a[2] = UserList[i].Password;
                a[3] = UserList[i].Level.ToString();
                a[4] = UserList[i].DateSigned.ToString();
                listView1.Items.Add(new ListViewItem(a));
            }
            Log("User List box updated.");
        }

        private void LoadCofiguration(String FileName)
        {
            ThisConfiguration = (ServerConfiguration)Serializer.LoadObject(FileName, Serializer.FormatterType.SOAP);
            if (ThisConfiguration == null)
            {
                ThisConfiguration = new ServerConfiguration();
                Serializer.SaveObject(ThisConfiguration, FileName, Serializer.FormatterType.SOAP);
            }
        }

        private void SaveDefaultUserList(String FileName)
        {
            List<User> a = new List<User>();
            a.Add(new User("Admin", "admin@ciao.server", "admin", UserLevel.Administrator, DateTime.Now));
            Serializer.SaveObject(a, FileName, Serializer.FormatterType.Binary);
        }

        private void SaveUserList(String FileName)
        {
            Serializer.SaveObject(UserList, FileName, Serializer.FormatterType.Binary);
            Log("User List saved to " + FileName + ".");
        }

        private void LoadUserList(String FileName)
        {
            UserList.RemoveRange(0, UserList.Count);
            this.UserList = (List<User>)Serializer.LoadObject(FileName, Serializer.FormatterType.Binary);
            if (this.UserList == null)
            {
                SaveDefaultUserList(FileName);
                this.UserList = (List<User>)Serializer.LoadObject(FileName, Serializer.FormatterType.Binary);
            }
            Log("User List loaded from " + FileName + ".");
        }

        private void MainServerForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            SaveUserList("UserList.bin");
            Serializer.SaveObject(ThisConfiguration, "conf.xml", Serializer.FormatterType.SOAP);
        }

        private void UpdateTimeString()
        {
            this.toolStripStatusLabel3.Text = DateTime.Now.ToString();
        }

        private void TimeUpdateTimer_Tick(object sender, EventArgs e)
        {
            UpdateTimeString();
            this.label3.Text = "Queue size: " + Convert.ToString(CiaoServer.Queue.Count);
        }

        private void ReportSendTimer_Tick(object sender, EventArgs e)
        {
            if (this.ThisConfiguration.ReportError)
            {
                String[] FileNames;
                try { FileNames = Directory.GetFiles("Exceptions"); }
                catch { return; }
                if (FileNames.Length > 0)
                {
                    String UserName = null;
                    String Password = null;
                    if (this.ThisConfiguration.UseAuthentication)
                    {
                        UserName = this.ThisConfiguration.UserName;
                        Password = this.ThisConfiguration.Password;
                    }
                    Report.SendReportToMail(FileNames[0], "bookysmell2004@yahoo.gr", this.ThisConfiguration.EMail, this.ThisConfiguration.SmtpServer, UserName, Password);
                }
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            String fileName;
            SaveLogDialog.ShowDialog(this);
            fileName = SaveLogDialog.FileName;
            if (SaveFileDialogOkPressed)
            {
                StreamWriter Writer = new StreamWriter(fileName);
                Writer.Write(this.textBox4.Text);
                Writer.Close();
                SaveFileDialogOkPressed = false;
            }
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            SaveFileDialogOkPressed = true;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (!CiaoServer.IsRunning)
            {
                CiaoServer.Port = Convert.ToInt32(this.textBox1.Text);
                try { CiaoServer.Start(); }
                catch { MessageBox.Show(this, "A server is already running on this machine. Only one usage of each socket address (protocol/network address/port) is normally permitted.", "Error"); };
                Log("Ciao server started on port " + Convert.ToString(CiaoServer.Port) + ".");
            }
            else
            {
                int Size = CiaoServer.ClientSockets.Count;
                CiaoServer.Stop();
                Log("Forcing disconnection of all clients.");
                Log(Convert.ToString(Size) + " clients were disconnected.");
                UpdateClientListBox();
                Log("Ciao server stopped.");
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            if (!ReportServer.IsRunning)
            {
                ReportServer.Port = Convert.ToInt32(this.textBox2.Text);
                try { ReportServer.Start(); }
                catch { MessageBox.Show(this, "A server is already running on this machine. Only one usage of each socket address (protocol/network address/port) is normally permitted.", "Error"); };
                Log("Report server started on port " + Convert.ToString(ReportServer.Port) + ".");
            }
            else
            {
                ReportServer.Stop();
                Log("Report server stopped.");
            }
        }

        void CiaoServer_ClientChange(object sender, ClientEventArgs e)
        {
            if (e.ClientState == ClientEvent.Connected)
            {
                ClientList.Add(new ConnectedClient(e.ClientSocket));
                Log("Established connection with " + e.ClientSocket.RemoteEndPoint.ToString() + ".");
                UpdateClientListBox();
                if (this.ThisConfiguration.RequestRsaEncryption)
                {
                    RSACryptoServiceProvider Provider = new RSACryptoServiceProvider();
                    ClientList[ClientList.Count - 1].TempKey = Provider.ToXmlString(true);
                    PacketData ToSend = new PacketData();
                    ToSend.DataCategory = "Encryption";
                    ToSend.DataType = "Request";
                    ToSend.MainData = Provider.ToXmlString(false);
                    SendEncrypted(new Packet(e.ClientSocket, ToSend));
                }
                return;
            }
            else
            {
                for (int i = 0; i < ClientList.Count; i++)
                {
                    if (ClientList[i].ConnectedSocket == e.ClientSocket)
                    {
                        ClientList.RemoveAt(i);
                        Log("Connection with " + e.ClientSocket.RemoteEndPoint.ToString() + " closed.");
                        UpdateClientListBox();
                        return;
                    }
                }
            }
        }

        private void UpdateClientListBox()
        {
            String[] a = new String[4];
            this.listView2.Items.Clear();
            for (int i = 0; i < ClientList.Count; i++)
            {
                a[0] = this.ClientList[i].ConnectedSocket.RemoteEndPoint.ToString();
                a[0] = a[0].Substring(0, a[0].IndexOf(':'));
                a[1] = this.ClientList[i].ConnectedSocket.RemoteEndPoint.ToString();
                a[1] = a[1].Substring(a[1].IndexOf(':') + 1);
                a[2] = this.ClientList[i].ConnectedUser.eMail;
                a[3] = this.ClientList[i].EncryptedConnection;
                this.listView2.Items.Add(new ListViewItem(a));
            }
            Log("Client List box updated.");
        }

        private void SendEncrypted(Packet p)
        {
            p.Buffer = Encryption.Encrypt(p.Buffer, 1024, ClientList.Find(delegate(ConnectedClient t) { return t.ConnectedSocket == p.StateSocket; }).PublicKey);
            CiaoServer.Send(p);
        }

        private ConnectedClient ReturnConnectedClient(System.Net.Sockets.Socket s)
        {
            return ClientList.Find(delegate(ConnectedClient t) { return t.ConnectedSocket == s; });
        }

        private List<Packet> PacketHandling(Packet NewPacket)
        {
            List<Packet> Table = new List<Packet>();
            String key; String pukey;

            //Decrypt packet
            key = ReturnConnectedClient(NewPacket.StateSocket).PrivateKey;
            NewPacket.Buffer = Encryption.Decrypt(NewPacket.Buffer, 1024, key);

            //Keep public key in case it's needed
            pukey = ReturnConnectedClient(NewPacket.StateSocket).PublicKey;

            //Get sent object
            PacketData ObjectSent = (PacketData)NewPacket.ObjectBuffer;
            PacketData ObjectToSend = new PacketData();

            //Proceed & fill Table
            #region Encryption
            if (ObjectSent.DataCategory == "Encryption")
            {
                if (ObjectSent.DataType == "Request")
                {
                    if (this.ThisConfiguration.AllowRsaEncryption)
                    {
                        ReturnConnectedClient(NewPacket.StateSocket).PublicKey = (String)ObjectSent.MainData;
                        ObjectToSend.DataCategory = "Encryption";
                        ObjectToSend.DataType = "Confirmation";
                        ObjectToSend.MainData = ObjectSent.MainData;
                        Table.Add(new Packet(NewPacket.StateSocket, ObjectToSend));
                    }
                }
                else if (ObjectSent.DataType == "Confirmation")
                {
                    ReturnConnectedClient(NewPacket.StateSocket).PrivateKey = ReturnConnectedClient(NewPacket.StateSocket).TempKey;
                }
            }
            #endregion
            #region User
            else if (ObjectSent.DataCategory == "User")
            {
                if (ObjectSent.DataType == "Request")
                {
                    if (ObjectSent.DataParameters == "SignUp")
                    {
                        UserLevel Level;
                        if ((String)(((String[])ObjectSent.MainData).GetValue(2)) == "User")
                        {
                            Level = UserLevel.User;
                        }
                        else if ((String)(((String[])ObjectSent.MainData).GetValue(2)) == "Administrator")
                        {
                            Level = UserLevel.Administrator;
                        }
                        else
                        {
                            Level = UserLevel.Anonymous;
                        }
                        User NewUser = new User((String)((String[])ObjectSent.MainData).GetValue(0), (String)((String[])ObjectSent.MainData).GetValue(0), (String)((String[])ObjectSent.MainData).GetValue(1), Level, DateTime.Now);
                        ObjectToSend.DataCategory = "User";
                        ObjectToSend.DataParameters = "SignUp";
                        ObjectToSend.MainData = NewUser.eMail;
                        if (NewUser.Level == UserLevel.Anonymous || (ReturnConnectedClient(NewPacket.StateSocket).ConnectedUser.Level != UserLevel.Administrator && NewUser.Level == UserLevel.Administrator) || (UserList.Find(delegate(User t) { return t.eMail == NewUser.eMail; }) != null))
                        {
                            ObjectToSend.DataType = "Rejection";
                        }
                        else
                        {
                            ObjectToSend.DataType = "Confirmation";
                            UserList.Add(NewUser);
                            UpdateUserListBox();
                        }
                        Table.Add(new Packet(NewPacket.StateSocket, ObjectToSend));
                    }
                    else if (ObjectSent.DataParameters == "LogIn")
                    {
                        String email = (String)((String[])ObjectSent.MainData).GetValue(0);
                        String password = (String)((String[])ObjectSent.MainData).GetValue(1);
                        ObjectToSend.DataCategory = "User";
                        ObjectToSend.DataParameters = "LogIn";
                        ObjectToSend.MainData = email;
                        if (UserList.Find(delegate(User t) { return t.eMail == email && t.Password == password; }) == null)
                        {
                            ObjectToSend.DataType = "Rejection";
                        }
                        else
                        {
                            ObjectToSend.DataType = "Confirmation";
                            ReturnConnectedClient(NewPacket.StateSocket).ConnectedUser = UserList.Find(delegate(User t) { return t.eMail == email && t.Password == password; });
                            PacketData ExtraObjectToSend = new PacketData();
                            ExtraObjectToSend.DataCategory = "Users";
                            ExtraObjectToSend.DataType = "Notification";
                            ExtraObjectToSend.DataParameters = "LogIn";
                            ExtraObjectToSend.MainData = email;
                            foreach (ConnectedClient t in ClientList)
                            {
                                Table.Add(new Packet(t.ConnectedSocket, ExtraObjectToSend));
                            }
                        }
                        Table.Add(new Packet(NewPacket.StateSocket,ObjectToSend));
                    }
                    else if (ObjectSent.DataParameters == "LogOut")
                    {
                        ObjectToSend.DataCategory = "User";
                        ObjectToSend.DataParameters = "LogOut";
                        ObjectToSend.DataType = "Confirmation";
                        Table.Add(new Packet(NewPacket.StateSocket,ObjectToSend));
                        ReturnConnectedClient(NewPacket.StateSocket).ConnectedUser = new User("Anonymous", "Anonymous@ciao.server", "", DateTime.Now);
                        PacketData ExtraObjectToSend = new PacketData();
                        ExtraObjectToSend.DataCategory = "Users";
                        ExtraObjectToSend.DataType = "Notification";
                        ExtraObjectToSend.DataParameters = "LogOut";
                        ExtraObjectToSend.MainData = ReturnConnectedClient(NewPacket.StateSocket).ConnectedUser.eMail;
                        foreach (ConnectedClient t in ClientList)
                        {
                            Table.Add(new Packet(t.ConnectedSocket, ExtraObjectToSend));
                        }
                    }
                }
            }
            #endregion
            #region Users
            else if (ObjectSent.DataCategory == "Users")
            {
                if (ObjectSent.DataType == "Request" && ObjectSent.DataParameters == "List")
                {
                    ObjectToSend.DataCategory = "Users";
                    ObjectToSend.DataParameters = "List";
                    if (ReturnConnectedClient(NewPacket.StateSocket).ConnectedUser == null)
                    {
                        ObjectToSend.DataType = "Rejection";
                    }
                    else
                    {
                        List<String> NameList = new List<String>();
                        ObjectToSend.DataType = "Answer";
                        foreach (ConnectedClient t in ClientList)
                        {
                            if (t.ConnectedUser != null) { NameList.Add(t.ConnectedUser.eMail); }
                        }
                        ObjectToSend.MainData = NameList.ToArray();
                    }
                    Table.Add(new Packet(NewPacket.StateSocket, ObjectToSend));
                }
            }
            #endregion
            #region Chat
            else if (ObjectSent.DataCategory == "Chat")
            {
                if (ObjectSent.DataType == "Request" && ObjectSent.DataParameters == "Send")
                {
                    ObjectToSend.DataCategory = "Chat";
                    ObjectToSend.DataType = "Notification";
                    ObjectToSend.MainData = new String[2] { ReturnConnectedClient(NewPacket.StateSocket).ConnectedUser.eMail, (String)ObjectSent.MainData };
                    foreach (ConnectedClient t in ClientList)
                    {
                        Table.Add(new Packet(t.ConnectedSocket, ObjectToSend));
                    }
                }
            }
            #endregion

            //Unknown command
            if (Table.Count == 0)
            {
                ObjectToSend.DataCategory = "OK";
                ObjectToSend.DataType = "OK";
                ObjectToSend.MainData = "OK";
                Table.Add(new Packet(NewPacket.StateSocket, ObjectToSend));
            }

            //Encrypt packets
            foreach (Packet p in Table)
            {
                if (p.StateSocket == NewPacket.StateSocket)
                {
                    key = pukey;
                }
                else
                {
                    key = ReturnConnectedClient(p.StateSocket).PublicKey;
                }
                p.Buffer = Encryption.Encrypt(p.Buffer, 1024, key);
            }

            //Send
            UpdateClientListBox();
            return Table;
        }

        private List<Packet> ReportPacketHandling(Packet NewPacket)
        {
            try
            {
                Report.SaveReport((Exception)NewPacket.ObjectBuffer);
            }
            catch { }
            return new List<Packet>();
        }
    }
}