
#include "stdafx.h"

#include "MailSlot.h"
#include "Package.h"

// 32K is the largest buffer we currently support. This number could be made
// larger if neccessary.
const int MAX_MESSAGE_SIZE = 32768;  

const CString MSG_TYPE = "MSG_TYPE";
const CString MAILSLOT_TO_NAME = "TO_MAILSLOT_NAME";
const CString MAILSLOT_TO_COMPUTER_NAME = "TO_COMPUTER_NAME";
const CString MAILSLOT_FROM_USER = "FROM_USER";
const CString MAILSLOT_FROM_COMPUTER = "FROM_COMPUTER";
const CString MAILSLOT_MSGTEXT = "MSGTEXT";

////////////////////////////////////////////////////////////////////////////////
// MailSlotMessage
//  A Package derived mailslot class. This class creates the basic message 
//  format for sending messages using the class MailSlot. 
//
MailSlotMessage::MailSlotMessage() : Package()
{
    MsgType(TEXT);
}

MailSlotMessage::MailSlotMessage(const MailSlotMessage & msg) : Package()
{
    Copy(msg);
}

MailSlotMessage & MailSlotMessage::operator=(const MailSlotMessage & msg)
{
    Copy(msg);
    return *this;
}

MailSlotMessage::MessageType MailSlotMessage::MsgType()
{
    int t = 0;
    Get(MSG_TYPE, t);
    MessageType type = (MessageType)t;
    return type;
}

void MailSlotMessage::MsgType(MailSlotMessage::MessageType type)
{
    int t = (int)type;
    Set(MSG_TYPE, t);
}

CString MailSlotMessage::ToUserName()
{
    CString toStr = "";
    Get(MAILSLOT_TO_NAME, toStr);
    return toStr;
}

void MailSlotMessage::ToUserName(const CString & mailSlotNameStr)
{
    Set(MAILSLOT_TO_NAME, mailSlotNameStr);
}

CString MailSlotMessage::ToComputerName()
{
    CString toStr = "";
    Get(MAILSLOT_TO_COMPUTER_NAME, toStr);
    return toStr;
}

void MailSlotMessage::ToComputerName(const CString & computerNameStr)
{
    Set(MAILSLOT_TO_COMPUTER_NAME, computerNameStr);
}

CString MailSlotMessage::FromUserName()
{
    CString fromUser = "";
    Get(MAILSLOT_FROM_USER, fromUser);
    return fromUser;
}

void MailSlotMessage::FromUserName(const CString & fromUser)
{
    Set(MAILSLOT_FROM_USER, fromUser);
}

CString MailSlotMessage::FromComputerName()
{
    CString fromComputer = "";
    Get(MAILSLOT_FROM_COMPUTER, fromComputer);
    return fromComputer;
}

void MailSlotMessage::FromComputerName(const CString & fromComputer)
{
    Set(MAILSLOT_FROM_COMPUTER, fromComputer);
}

CString MailSlotMessage::MsgText()
{
    CString msgText = "";
    Get(MAILSLOT_MSGTEXT, msgText);
    return msgText;
}

void MailSlotMessage::MsgText(const CString & msgText)
{
    Set(MAILSLOT_MSGTEXT, msgText);
}


MailSlot::MessageQueue::MessageQueue()
{

}

MailSlot::MessageQueue::~MessageQueue()
{
    Reset();
}

// Add a message to the queue. 
void MailSlot::MessageQueue::Add(MailSlotMessage & msg)
{
    msgAry.Add(msg);
}

// Get message from queue. This method will also remove the message
// from the queue.
bool MailSlot::MessageQueue::Get(MailSlotMessage & msg)
{
    if (Count() > 0) {
        msg = msgAry.GetAt(0);
        msgAry.RemoveAt(0);
        return true;
    }
    return false;
}

// Returns number of messages in the queue.
int MailSlot::MessageQueue::Count()
{
    return (int)msgAry.GetSize();
}

// Remove all messages from the queue.
void MailSlot::MessageQueue::Reset()
{
    msgAry.RemoveAll();
}

////////////////////////////////////////////////////////////////////////////////
// MailSlot

BEGIN_MESSAGE_MAP(MailSlot, CWnd)
    ON_WM_TIMER()
END_MESSAGE_MAP()

MailSlot::MailSlot()
{
    pParentWnd = NULL;
    m_notifyID = 0;
}

MailSlot::~MailSlot()
{
    if (mailSlotHandle) {
        CloseHandle(mailSlotHandle);
        mailSlotHandle = NULL;
    }
}

BOOL MailSlot::Create(const CString & mailSlotName, CWnd * pParent, UINT notifyID /*=0*/, UINT pollTime /*=1000*/)
{
    m_notifyID = notifyID;
    slotName = mailSlotName;
    slotPath.Format("\\\\.\\mailslot\\%s", slotName);

    mailSlotHandle = CreateMailslot(slotPath,
        0,
        MAILSLOT_WAIT_FOREVER,
        NULL);
    if (mailSlotHandle != INVALID_HANDLE_VALUE) {
        pParentWnd = pParent;
        CRect r;
        CWnd::CreateEx(0, NULL, "MailSlot", WS_CHILD, r, pParentWnd, 0, NULL);
        m_nTimer = CWnd::SetTimer(1, pollTime, NULL);
        char computerNameStr[256];
        DWORD computerNameLen = 256;
        GetComputerName(computerNameStr, &computerNameLen);
        computerName = computerNameStr;
        return TRUE;
    }

    return FALSE;
}

BOOL MailSlot::Read(MailSlotMessage & msg)
{
    if (queue.Count() > 0) {
        return queue.Get(msg);
    }
    return FALSE;
}

BOOL MailSlot::Send(MailSlotMessage & msg)
{       
    if (msg.ToComputerName() == computerName && msg.ToUserName() == slotName) {
        return SendToLocal(msg);
    } else {
        return SendToMailSlot(msg);
    }
}

BOOL MailSlot::SendToLocal(MailSlotMessage & msg)
{
    queue.Add(msg);
    NotifyParent();
    return TRUE;
}

BOOL MailSlot::SendToMailSlot(MailSlotMessage & msg)
{
    BOOL ret = TRUE;
    CString dest;
    dest.Format("\\\\%s\\mailslot\\%s", msg.ToComputerName(), msg.ToUserName());
    HANDLE handle = CreateFile(dest, GENERIC_WRITE, 
        FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0); 
    if (handle == INVALID_HANDLE_VALUE) { 
        TRACE("CreateFile failed");
        ret = FALSE;;
    } 
    if (ret && handle) {
        BYTE * pBuf;
        PackSize size = msg.ToBuffer(pBuf);

        DWORD bytesWritten;
        ret = WriteFile(handle, pBuf, size, &bytesWritten, 0);

        if (bytesWritten != size)
            ret = FALSE;
        
        delete[] pBuf;
    }
    if (handle && handle != mailSlotHandle)
        CloseHandle(handle);

    return ret;
}

BOOL MailSlot::ReadFromMailSlot()
{
    if (!mailSlotHandle)
        return FALSE;

    MailSlotMessage msg;

    DWORD waitTime = 0;
    DWORD msgSize = MAX_MESSAGE_SIZE;

    BOOL ret = GetMailslotInfo(mailSlotHandle, 0, &msgSize, 0, &waitTime);
    if (msgSize == (DWORD)MAILSLOT_NO_MESSAGE) 
        ret = FALSE;

    PackSize bytesLeft = msgSize;
    while(ret && bytesLeft > 0) {
        BYTE * pBuf;
        if (msgSize > MAX_MESSAGE_SIZE)
            msgSize = MAX_MESSAGE_SIZE;
        pBuf = NULL;
        pBuf = (BYTE*)GlobalAlloc(GMEM_FIXED, msgSize);
        if (!pBuf) 
            ASSERT("An error getting a memory block." == 0);
        else {
            DWORD numRead;
            ret = ReadFile(mailSlotHandle, pBuf, bytesLeft, &numRead, 0);
            if (ret && numRead > 0)
                bytesLeft -= msg.FromBuffer(pBuf, (PackSize)bytesLeft);
        }
        GlobalFree(pBuf);
        MailSlotMessage copyMsg = msg;
        if (ret)
            queue.Add(msg);
    }
    return ret;
}

void MailSlot::OnTimer(UINT_PTR nIDEvent)
{
    if (ReadFromMailSlot())
        NotifyParent();
}

void MailSlot::NotifyParent()
{
    if (pParentWnd && pParentWnd->m_hWnd && m_notifyID)
        ::PostMessage(pParentWnd->m_hWnd, m_notifyID, 0, 0);
}